// services/api/user.ts
import Taro from '@tarojs/taro'
import { ensureStringId } from '@/utils/db-helper'

// 明确定义接口类型
export interface UserInfo {
  openid: string;
  nickname?: string;
  avatar?: string;
  role?: string;
}

export interface OpenIdResult {
  openid: string;
  [key: string]: any;
}

export interface ApiResult<T> {
  success: boolean;
  data?: T;
  message?: string;
}

// 控制是否使用云函数(true)或mock数据(false)
const USE_CLOUD = true

// Mock用户数据
const mockUser = {
  nickname: '测试用户',
  avatar: 'https://example.com/avatar.jpg',
  openid: 'mock_openid_123456',
}

export const userAPI = {
  // 获取用户信息
  async getUserInfo(): Promise<ApiResult<UserInfo>> {
    try {
      if (USE_CLOUD && process.env.TARO_ENV === 'weapp') {
        // 云函数实现
        const result = await Taro.cloud.callFunction({
          name: 'getUserInfo',
        })
        
        // 使用类型断言处理结果
        if (result && typeof result === 'object' && 'result' in result) {
          const cloudResult = result.result as any
          return { 
            success: true, 
            data: cloudResult?.data || mockUser 
          }
        }
        return { success: false, message: '获取用户信息失败' }
      } else {
        // Mock实现
        return new Promise(resolve => {
          setTimeout(() => {
            resolve({ success: true, data: mockUser })
          }, 300)
        })
      }
    } catch (error) {
      console.error('获取用户信息失败:', error)
      return { success: false, message: error.message || '获取用户信息失败' }
    }
  },
  
  // 获取OpenID
  async getOpenId(code: string): Promise<ApiResult<OpenIdResult>> {
    try {
      if (USE_CLOUD && process.env.TARO_ENV === 'weapp') {
        const result = await Taro.cloud.callFunction({
          name: 'getOpenId',
          data: { code }
        })
        
        if (result && result.result) {
          return { success: true, data: result.result as OpenIdResult }
        }
        return { success: false, message: '获取OpenID失败' }
      }
      
      // Mock实现
      return { 
        success: true, 
        data: { openid: 'mock_openid_' + Date.now() } 
      }
    } catch (error) {
      console.error('获取OpenID失败:', error)
      return { success: false, message: error.message || '获取OpenID失败' }
    }
  },
  
  // 保存用户信息到云端
  async saveUserToCloud(userData: UserInfo): Promise<ApiResult<string>> {
    try {
      if (!USE_CLOUD || process.env.TARO_ENV !== 'weapp') {
        return { success: true, data: 'mock_doc_id' } // Mock成功
      }
      
      // 确保云已初始化
      try {
        await Taro.cloud.init({
          env: (Taro.cloud as any).DYNAMIC_CURRENT_ENV
        })
      } catch (initError) {
        console.warn('云初始化可能已完成:', initError)
        // 继续执行，即使初始化报错
      }
      
      const db = Taro.cloud.database()
      
      // 查询用户是否存在
      const userQuery = await db.collection('users').where({
        openid: userData.openid
      }).get()
      
      // 如果用户已存在，更新信息
      if (userQuery.data && userQuery.data.length > 0) {
        // 修复: 使用类型断言确保 _id 转换为字符串
        const docId = ensureStringId(userQuery.data[0]._id)
        
        await db.collection('users').doc(docId).update({
          data: {
            nickname: userData.nickname,
            avatar: userData.avatar,
            role: userData.role,
            updateTime: db.serverDate()
          }
        })
        return { success: true, data: docId }
      } else {
        // 创建新用户
        const addResult = await db.collection('users').add({
          data: {
            ...userData,
            createTime: db.serverDate(),
            updateTime: db.serverDate()
          }
        })
        
        // 修复: 确保返回的 _id 是字符串
        const newId = typeof addResult._id === 'number' 
          ? String(addResult._id) 
          : addResult._id as string
          
        return { success: true, data: newId }
      }
    } catch (error) {
      console.error('保存用户信息到云端失败:', error)
      return { success: false, message: error.message || '保存用户信息失败' }
    }
  }
}