# LineJoy 小程序项目架构指南（更新版）

本文档提供对 LineJoy 项目中 services 和 stores 目录的使用说明，阐明职责分离原则和最佳实践，帮助开发者理解项目架构并促进协作开发。

## 目录

1. [整体架构](#整体架构)
2. [架构核心原则](#架构核心原则)
3. [Services 服务层](#services-服务层)
4. [Stores 状态管理层](#stores-状态管理层)
5. [协同工作流程](#协同工作流程)
6. [扩展指南](#扩展指南)
7. [最佳实践](#最佳实践)

## 整体架构

LineJoy 项目采用了模块化的架构设计，主要分为以下几层：

- **Services 层**：负责与外部 API 通信、数据处理，提供纯粹的数据操作接口
- **Stores 层**：使用 Pinia 管理应用状态，封装业务逻辑，提供给组件使用
- **Components 层**：UI 组件，消费 stores 中的状态
- **Pages 层**：页面组件，组合多个组件形成完整的页面

## 架构核心原则

项目架构遵循以下核心原则：

### 1. Store-First 模式

**组件应该优先与 Store 交互，而不是直接调用 API**。这样可以确保：
- 统一的状态管理
- 自动的缓存机制
- 响应式更新
- 集中的业务逻辑

### 2. 明确的职责分离

- **Services/API 层**：纯粹的数据操作和外部通信
  - 不直接修改应用状态
  - 不包含业务逻辑
  - 仅提供数据获取和操作函数
  - 返回原始数据，不处理状态转换

- **Stores 层**：状态管理和业务逻辑
  - 调用服务方法获取数据
  - 管理应用状态
  - 处理业务逻辑
  - 提供计算属性和操作方法给组件使用

### 3. 统一的数据源

确保应用内数据一致性：
- 使用中央存储的状态
- 避免重复调用 API
- 通过 Store 共享数据
- 统一的数据刷新策略

### 4. 统一的前置检查

在 Store 层面统一处理常见检查，如：
- 用户注册状态检查
- 权限验证
- 输入验证
- 业务规则验证

## Services 服务层

服务层处理所有与外部系统的交互，包括 API 调用、云函数调用、本地存储等。

### 职责范围

- 提供纯粹的数据获取和操作方法
- 不关心应用状态
- 不直接与 UI 交互
- 处理 API 调用、错误处理和重试逻辑
- 可选地提供数据转换，但不处理业务逻辑

### 目录结构

```
src/services/
├── api/                # API 服务
│   ├── game.ts         # 游戏列表和详情 API
│   ├── room.ts         # 房间管理 API
│   ├── user.ts         # 用户管理 API
│   └── games/          # 特定游戏类型 API
│       ├── index.ts    # 游戏 API 索引
│       └── turtlesoup.ts # 海龟汤游戏 API
├── cloud.ts            # 云函数调用封装
└── request.ts          # 通用 HTTP 请求封装
```

### 服务层设计原则

1. **独立性**：每个服务模块应该独立工作，不依赖其他服务
2. **纯函数**：API 方法应尽量设计为纯函数，输入确定则输出确定
3. **完整返回**：返回完整的响应对象，包括成功/失败状态和数据
4. **错误处理**：在服务层处理异常并返回统一格式的错误信息
5. **Mock 支持**：为开发和测试环境提供 mock 数据支持

### 使用示例

```typescript
// services/api/room.ts
export const roomAPI = {
  // 检查房间是否存在
  async checkRoomExist(roomId: string): Promise<{exists: boolean, needPassword: boolean}> {
    try {
      // 实现房间检查逻辑
      return { exists: true, needPassword: false }
    } catch (error) {
      console.error('检查房间失败:', error)
      return { exists: false, needPassword: false }
    }
  },
  
  // 加入房间 - 纯粹的数据操作，不依赖store
  async joinRoom(roomId: string, userData: RoomUser): Promise<{success: boolean, roomData?: Room, message?: string}> {
    try {
      // 实现加入房间逻辑
      return { success: true, roomData: { /* 房间数据 */ } }
    } catch (error) {
      console.error('加入房间失败:', error)
      return { success: false, message: '加入房间失败' }
    }
  }
}
```

## Stores 状态管理层

状态管理层使用 Pinia 管理应用状态，封装业务逻辑，提供给组件使用。

### 职责范围

- 管理应用状态
- 封装业务逻辑
- 调用服务方法获取和操作数据
- 提供计算属性和操作方法给组件使用
- 处理前置检查和验证
- 维护数据一致性
- 管理UI状态（如加载状态、错误状态）

### 目录结构

```
src/stores/
├── game.ts             # 游戏列表和详情状态
├── room.ts             # 房间管理状态
├── user.ts             # 用户信息和认证状态
├── tabbar.ts           # 导航栏状态
└── games/              # 特定游戏类型状态
    ├── index.ts        # 游戏状态索引
    └── turtlesoup.ts   # 海龟汤游戏状态
```

### 状态层设计原则

1. **单一职责**：每个 Store 应专注于单一领域的状态管理
2. **业务逻辑封装**：在 Store 中封装业务逻辑，而不是在组件中
3. **前置检查**：统一处理前置检查（如用户注册状态）
4. **状态统一性**：确保相关状态在各个组件中的一致性
5. **智能导航**：基于状态(如房间状态)决定导航行为

### 使用示例

```typescript
// stores/room.ts
export const useRoomStore = defineStore('room', () => {
  // 状态定义
  const currentRoom = ref<Room | null>(null)
  const recentRooms = ref<RecentRoom[]>([])
  
  // 统一的用户注册检查
  async function checkUserRegistered(): Promise<{valid: boolean, message?: string}> {
    if (!userStore.isRegistered) {
      return { valid: false, message: '需要先完善个人信息才能继续操作' }
    }
    return { valid: true }
  }
  
  // 加入房间 - 包含业务逻辑和前置检查
  async function joinRoom(roomId: string) {
    try {
      // 前置检查 - 用户是否已注册
      const userCheck = await checkUserRegistered()
      if (!userCheck.valid) {
        return { success: false, message: userCheck.message }
      }
      
      // 构建用户数据
      const userData = {
        id: userStore.openid,
        name: userStore.nickname,
        // ... 其他用户数据
      }
      
      // 调用 API 加入房间
      const result = await roomAPI.joinRoom(roomId, userData)
      
      // 更新状态
      if (result.success && result.roomData) {
        currentRoom.value = result.roomData
        // ... 其他状态更新
      }
      
      return result
    } catch (error) {
      console.error('加入房间失败:', error)
      return { success: false, message: '加入房间失败' }
    }
  }
  
  // 加入房间的便捷方法 - 处理完整流程
  async function joinRoomById(roomId: string, password?: string) {
    // 实现完整的加入房间流程，包括验证房间、检查密码等
    // ...
    
    return await joinRoom(roomId)
  }
  
  // 智能导航方法 - 根据房间状态决定导航行为
  function navigateToRoomPage(roomId: string) {
    // 根据房间状态决定导航目标
    if (currentRoom.value?.status === 'playing') {
      Taro.navigateTo({ url: `/pages/room/play/index?roomId=${roomId}` })
    } else {
      Taro.navigateTo({ url: `/pages/room/waiting/index?roomId=${roomId}` })
    }
  }
  
  // 返回状态和方法
  return {
    currentRoom,
    recentRooms,
    joinRoom,
    joinRoomById,
    navigateToRoomPage,
    // ... 其他方法
  }
})
```

## 协同工作流程

在 Store-First 模式中，Services 和 Stores 的协同工作流程如下：

### 标准数据流

```
组件 -> Store -> Service -> 外部 API/数据源 -> Service -> Store -> 组件
```

1. **组件触发操作**：用户在 UI 组件上触发操作（如点击按钮）
2. **调用 Store 方法**：组件调用相应 Store 的方法处理操作
3. **前置检查**：Store 进行前置检查（如用户注册状态）
4. **调用 Service**：Store 调用适当的 Service 方法获取或操作数据
5. **处理响应**：Store 处理 Service 返回的结果并更新状态
6. **状态更新通知**：由于 Store 状态变化，相关组件自动重新渲染
7. **智能导航**：根据操作结果和状态，Store 可能指导组件进行页面导航

### 组件级实现示例

```vue
<script lang="ts">
import { useRoomStore } from '@/stores/room'

export default {
  setup() {
    const roomStore = useRoomStore()
    
    // 加入房间方法
    const joinRoom = async (roomId: string, password?: string) => {
      try {
        Taro.showLoading({ title: '加入房间中...' })
        
        // 调用 Store 方法 - 处理完整的加入流程
        const result = await roomStore.joinRoomById(roomId, password)
        
        if (result.success) {
          // 使用 Store 的智能导航
          roomStore.navigateToRoomPage(roomId)
        } else {
          if (result.needPassword) {
            // 处理需要密码的情况
            // ...
          } else {
            Taro.showToast({ title: result.message || '加入房间失败', icon: 'none' })
          }
        }
      } catch (error) {
        console.error('加入房间失败:', error)
        Taro.showToast({ title: '加入房间失败', icon: 'none' })
      } finally {
        Taro.hideLoading()
      }
    }
    
    return {
      joinRoom,
      // ... 其他方法和状态
    }
  }
}
</script>
```

## 扩展指南

### 添加新的 Service

1. 在 `services/api` 目录下创建新的服务文件，如 `myService.ts`
2. 实现纯粹的数据获取和操作方法，不涉及状态管理
3. 确保错误处理和完整的返回值

```typescript
// services/api/myService.ts
export const myServiceAPI = {
  async getData(param: string): Promise<{success: boolean, data?: any, message?: string}> {
    try {
      // 实现数据获取逻辑
      return { success: true, data: { /* 数据 */ } }
    } catch (error) {
      console.error('获取数据失败:', error)
      return { success: false, message: '获取数据失败' }
    }
  }
}
```

### 添加新的 Store

1. 在 `stores` 目录下创建新的 store 文件，如 `myStore.ts`
2. 导入并使用相应的 service
3. 实现状态管理和业务逻辑

```typescript
// stores/myStore.ts
import { defineStore } from 'pinia'
import { myServiceAPI } from '@/services/api/myService'
import { useUserStore } from './user'

export const useMyStore = defineStore('myStore', () => {
  // 获取其他 store
  const userStore = useUserStore()
  
  // 状态定义
  const data = ref(null)
  const loading = ref(false)
  const error = ref(null)
  
  // 加载数据
  async function loadData(param: string) {
    // 前置检查
    if (!userStore.isLoggedIn) {
      return { success: false, message: '请先登录' }
    }
    
    loading.value = true
    error.value = null
    
    try {
      // 调用 service 获取数据
      const result = await myServiceAPI.getData(param)
      
      if (result.success && result.data) {
        // 更新状态
        data.value = result.data
      } else {
        error.value = result.message
      }
      
      return result
    } catch (e) {
      error.value = '加载数据失败'
      return { success: false, message: '加载数据失败' }
    } finally {
      loading.value = false
    }
  }
  
  // 返回状态和方法
  return {
    data,
    loading,
    error,
    loadData
  }
})
```

## 最佳实践

### 统一的数据获取模式

采用 Store-First 模式进行数据获取：

1. 组件首先检查 Store 中是否已有数据
2. 如果有数据且不需要刷新，直接使用 Store 中的数据
3. 如果没有数据或需要刷新，调用 Store 方法获取数据
4. Store 方法调用 Service 方法，并更新状态
5. 组件通过响应式绑定显示数据

```typescript
// 在组件中
const loadData = async () => {
  // 检查是否需要加载
  if (!dataStore.loaded || forceRefresh) {
    await dataStore.loadData()
  }
  
  // 直接使用 store 中的数据
  return dataStore.data
}
```

### 统一的错误处理

在 Store 层面统一处理错误：

1. Service 方法捕获和处理技术错误（如网络异常）
2. Store 方法处理业务错误（如权限不足）
3. 组件处理 UI 相关错误（如显示提示）

```typescript
// 在 Store 中
async function performAction() {
  try {
    // 前置检查
    if (!canPerformAction()) {
      return { success: false, code: 'PERMISSION_DENIED', message: '权限不足' }
    }
    
    // 调用 Service
    const result = await someAPI.doSomething()
    
    // 处理业务错误
    if (!result.success) {
      if (result.code === 'RATE_LIMITED') {
        return { success: false, message: '操作过于频繁，请稍后再试' }
      }
      return result
    }
    
    // 更新状态
    updateState(result.data)
    
    return { success: true, data: result.data }
  } catch (error) {
    logError('performAction', error)
    return { success: false, message: '操作失败，请重试' }
  }
}
```

### 智能导航

在 Store 层面实现基于状态的智能导航：

```typescript
// 在 Store 中
function navigateBasedOnState(id: string) {
  // 基于当前状态决定导航行为
  if (currentState.value === 'COMPLETED') {
    Taro.navigateTo({ url: `/pages/result/index?id=${id}` })
  } else if (currentState.value === 'IN_PROGRESS') {
    Taro.navigateTo({ url: `/pages/process/index?id=${id}` })
  } else {
    Taro.navigateTo({ url: `/pages/start/index?id=${id}` })
  }
}
```

### 缓存策略

在 Store 中实现智能缓存策略：

```typescript
// 在 Store 中
const cacheExpiry = ref<Record<string, number>>({})

async function getData(id: string, forceRefresh = false) {
  // 检查缓存是否有效
  const now = Date.now()
  const cachedTime = cacheExpiry.value[id] || 0
  const cacheValid = now - cachedTime < CACHE_TTL
  
  // 如果缓存有效且不强制刷新，直接返回缓存数据
  if (cacheValid && !forceRefresh && dataCache.value[id]) {
    return dataCache.value[id]
  }
  
  // 获取新数据
  const result = await dataAPI.getData(id)
  
  if (result.success) {
    // 更新缓存
    dataCache.value[id] = result.data
    cacheExpiry.value[id] = now
  }
  
  return result.data
}
```

## 结语

通过遵循 Store-First 模式和明确的职责分离原则，LineJoy 项目实现了清晰的架构和工作流程。这种架构不仅提高了代码的可维护性和可扩展性，也确保了数据的一致性和用户体验的流畅性。

在开发新功能时，请遵循本指南中的原则和最佳实践，保持项目的一致性和质量。
